@@ -16,11 +16,11 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_MYSQL
-static const char rcsid[] = "$Id: auth_mysql.c,v 1.21 2001/06/24 00:15:55 chris Exp $";
+static const char rcsid[] = "$Id: auth_mysql.c,v 1.24 2001/07/24 00:08:10 chris Exp $";
 
 #include <sys/types.h> /* BSD needs this here, apparently. */
 
-#ifdef CRYPT_FUNCTION_IN_CRYPT_H
+#ifdef HAVE_CRYPT_H
 #include <crypt.h>
 #endif
 
@@ -59,18 +59,18 @@ static const char rcsid[] = "$Id: auth_mysql.c,v 1.21 2001/06/24 00:15:55 chris
  * ----------------------------------------------------------------------------
  */
 
-static unsigned char itoa64[] =		/* 0 ... 63 => ascii - 64 */
-	"./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
+static unsigned char itoa64[] = /* 0 ... 63 => ascii - 64 */
+        "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
 
 /* to64:
  * Convert a string into a different base.
  */
 static void to64(char *s, unsigned long v, int n)
 {
-	while (--n >= 0) {
-		*s++ = itoa64[v&0x3f];
-		v >>= 6;
-	}
+    while (--n >= 0) {
+        *s++ = itoa64[v&0x3f];
+        v >>= 6;
+    }
 }
 
 /* crypt_md5:
@@ -83,9 +83,9 @@ static char *crypt_md5(const char *pw, const char *salt)
      * it this way, we can get get better later on */
     static char passwd[120], *p;
     static const char *sp,*ep;
-    unsigned char	final[16];
+    unsigned char   final[16];
     int sl,pl,i,j;
-    MD5_CTX	ctx,ctx1;
+    MD5_CTX ctx,ctx1;
     unsigned long l;
 
     /* Refine the Salt first */
@@ -264,9 +264,9 @@ const char apop_query_template[] =
        "AND popbox.domain_name = '%s' "
        "AND popbox.domain_name = domain.domain_name";
 authcontext auth_mysql_new_apop(const char *name, const char *timestamp, const unsigned char *digest) {
-    char *query, *x, *y;
+    char *query = NULL, *x = NULL, *y = NULL;
     authcontext a = NULL;
-    char *local_part;
+    char *local_part = NULL;
     const char *domain;
     item *I;
     int use_gid = 0;
@@ -411,9 +411,9 @@ char user_pass_query_template[] =
        "AND popbox.domain_name = '%s' "
        "AND popbox.domain_name = domain.domain_name";
 authcontext auth_mysql_new_user_pass(const char *user, const char *pass) {
-    char *query, *x, *y;
+    char *query = NULL, *x = NULL, *y = NULL;
     authcontext a = NULL;
-    char *local_part;
+    char *local_part = NULL;
     const char *domain;
     char *p;
     unsigned char *q;
