@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
  *
- * $Id: auth_other.h,v 1.1 2001/04/23 23:45:40 chris Exp $
+ * $Id: auth_other.h,v 1.2 2001/04/26 23:30:03 chris Exp $
  *
  */
 
@@ -29,7 +29,7 @@ int auth_other_start_child(void);
 void auth_other_kill_child(void);
 int auth_other_init(void);
 void auth_other_close(void);
-int auth_other_send_request(int nvars, ...);
+int auth_other_send_request(const int nvars, ...);
 stringmap auth_other_recv_response(void);
 authcontext auth_other_new_apop(const char *name, const char *timestamp, const unsigned char *digest);
 authcontext auth_other_new_user_pass(const char *user, const char *pass);
