@@ -11,16 +11,15 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_PASSWD
-static const char rcsid[] = "$Id: auth_passwd.c,v 1.10 2001/06/24 00:15:55 chris Exp $";
+static const char rcsid[] = "$Id: auth_passwd.c,v 1.11 2001/07/24 00:08:10 chris Exp $";
 
 #include <sys/types.h>
 
-#ifdef CRYPT_FUNCTION_IN_CRYPT_H    /* XXX */
+#ifdef HAVE_CRYPT_H /* XXX */
 #include <crypt.h>
-#else
-#include <unistd.h>
 #endif
 
+#include <unistd.h>
 #include <grp.h>
 #include <pwd.h>
 #ifdef AUTH_PASSWD_SHADOW
