@@ -9,7 +9,7 @@
  *
  */
 
-static const char rcsid[] = "$Id: cfgdirectives.c,v 1.5 2001/04/26 23:30:03 chris Exp $";
+static const char rcsid[] = "$Id: cfgdirectives.c,v 1.6 2001/07/08 16:57:09 chris Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -65,6 +65,10 @@ char *cfgdirectives[] = {
     "auth-perl-apop",
     "auth-perl-pass",
 #endif /* AUTH_PERL */
+ 
+#if defined(MBOX_BSD) && defined(MBOX_BSD_SAVE_INDICES)
+    "mailspool-index",
+#endif
     
     /* final entry must be NULL */
     NULL};
