@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
  *
- * $Id: util.h,v 1.13 2001/06/10 21:52:10 chris Exp $
+ * $Id: util.h,v 1.15 2001/07/24 00:08:10 chris Exp $
  *
  */
 
@@ -16,12 +16,17 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 
 #ifndef TPOP3D_VERSION
 #   define TPOP3D_VERSION   "(unknown)"
 #endif
 
-#define PAGESIZE        getpagesize()
+#ifndef PAGESIZE
+#   define PAGESIZE        getpagesize()
+#endif
 
 #if 0
 /* Primitive memory-leak debugging. */
@@ -36,6 +41,9 @@ void *myrealloc(char *f, int l, void *p, const size_t n);
 #define realloc(a, b)   myrealloc(__FILE__, __LINE__, a, b)
 #endif
 
+/* reallocating strncat. */
+char *xstrncat(char *pfx, const char *sfx, const size_t n);
+
 /* Function for substituting $(...) in strings. */
 struct sverr {
     char *msg;
@@ -70,6 +78,12 @@ int inet_aton(const char *s, struct in_addr *ip);
 char *strtok_r(char *s, const char *delim, char **saveptr);     /* GNU implementation in strtok_r.c */
 #endif
 
+/* Turn a 16-byte buffer into a binary string. */
+char *hex_digest(const unsigned char *u);
+
+/* Vice versa. */
+int unhex_digest(const char *from, unsigned char *to);
+
 /* Optional internationalisation support. */
 #ifdef WITH_I18N
 #   include <gettext.h>
